/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2013. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
 
(function(){
	dojo.provide("uml.rmps.rendering.RenderingAdminPage");
	
	dojo.require("dojo.data.ItemFileWriteStore");
	dojo.require("dojox.grid.DataGrid");

	dojo.require("net.jazz.ajax.ui.PlatformUI");
	dojo.require("uml.rmps.rendering.RenderingServerEditDialog");
	
	dojo.require("jazz.ui.StyledTable");
	dojo.require("jazz.ui.Toolbar");
	dojo.require("jazz.ui.toolbar.Button");
	dojo.require("jazz.ui.toolbar.Separator");
    dojo.require("jazz.util.html");
    
	dojo.require("dojo.i18n");
	dojo.requireLocalization("uml.rmps.rendering", "RenderingAdminPageMessages");
	
	dojo.declare("uml.rmps.rendering.RenderingAdminPageToolbar", jazz.ui.Toolbar, {
		 addButton: function addButton(button){
			this.addChild(button);
			 
			 this.connect(button, "_onFocus", dojo.hitch(button, function(e){ //$NON-NLS-1$
				 var row = this.domNode;
				 while (row && row.tagName.toLowerCase() !== "tr") { //$NON-NLS-1$
					 row = row.parentNode;
				 }
				 if (row) {
					 dojo.toggleClass(row, "selected", true); //$NON-NLS-1$
				 }
			 }));
			 this.connect(button, "_onBlur", dojo.hitch(button, function(e){ //$NON-NLS-1$
				 var row = this.domNode;
				 while (row && row.tagName.toLowerCase() !== "tr") { //$NON-NLS-1$
					 row = row.parentNode;
				 }
				 if (row) {
					 dojo.toggleClass(row, "selected", false); //$NON-NLS-1$
				 }
			 }));
			 
			 if (button.params.hasOwnProperty("rowNumber") && button.titleNode) {
				 button.titleNode.rowNumber = button.params.rowNumber;
			 }
		 }
	 });
	
	var messages = dojo.i18n.getLocalization("uml.rmps.rendering", "RenderingAdminPageMessages");
	 
	var RENDERING_SERVERS_URL = net.jazz.ajax._contextRoot + "/renderingservers";
	var SERVER_STATUS_URL = RENDERING_SERVERS_URL + "/status";
	
	var COLUMN_ID_NAME = "Name";
	var COLUMN_ID_URL = "URL";
	var COLUMN_ID_STATUS = "Status";
	var COLUMN_ID_ENABLED = "Enabled";
	var COLUMN_ID_ACTIONS = "Actions";
	
	var ONLINE_ICON = dojo.moduleUrl("rmps.core", "graphics/icons/state_completed.gif");
	var OFFLINE_ICON = dojo.moduleUrl("rmps.core", "graphics/icons/state_error.gif");
	
	
	dojo.declare("uml.rmps.rendering.RenderingAdminPage", [dijit._Widget, dijit._Templated], {
		
		templatePath: dojo.moduleUrl("uml.rmps", "rendering/templates/RenderingAdminPage.html"),

		constructor: function(args) {
			this.messages = messages;
			this.toolbars = [];
			this._servers = [];
	    },
	    
		postCreate: function() {
			this.inherited(arguments);
			var ActionRegistry =  net.jazz.ajax.ui.PlatformUI.getWorkbench().getActionRegistry();
			ActionRegistry.registerAction("uml.rmps.renderingAdmin", function() {});
			this._createTopControls();
			this.refresh(/* onlyTable */ false);
		},

		startup: function() {},
		
		_columns : [
				    {
						id: COLUMN_ID_NAME,
						label: messages.nameLabel,
						isEditable: true,
						width: "25%"
				    },
				    {
						id: COLUMN_ID_URL,
						label: messages.urlLabel,
						isEditable: true,
						width: "45%"
				    },
				    {
						id: COLUMN_ID_STATUS,
						label: messages.statusLabel
				    },
				    {
						id: COLUMN_ID_ENABLED,
						label: messages.enabledLabel,
						isEditable: true,
						checkbox: true
				    },
				    {
						id: COLUMN_ID_ACTIONS,
						label: messages.actionsLabel,
						width: "60px"
				    }
		],
		
		_updateStatus: function(node, status) {
			var isOnline = true;
			var messagesList = [];
			if (!status) {
				isOnline = false;
				messagesList.push(messages.serverStatusCannotConnectToExtension);
			} else {
				if (!status.dm || status.dm.code != 200) {
					isOnline = false;
					messagesList.push(messages.serverStatusCannotConnectToDM);
				}
				
				if (!status.vvc || status.vvc.code != 200) {
					isOnline = false;
					messagesList.push(messages.serverStatusCannotConnectToVVC);
				}
				
				if (!status.jts || status.jts.code != 200) {
					isOnline = false;
					messagesList.push(messages.serverStatusCannotConnectToJTS);
				}
			}
			dojo.empty(node);
			var messageText = isOnline ? messages.serverOnline : messages.serverOffline;

			node.appendChild(dojo.create("img", {
				alt: messageText,
				src: isOnline ? ONLINE_ICON : OFFLINE_ICON,
				align: "top"
			}));
			
			node.appendChild(dojo.doc.createTextNode(messageText));
			for (var i = 0; i < messagesList.length; ++i) {
				node.appendChild(dojo.create("br"));
				node.appendChild(dojo.doc.createTextNode(messagesList[i]));
			}
		},
		
		_checkStatus: function(resultNode, serverUrl) {
			var def = rmps.core.client.xhrGet({
				url: SERVER_STATUS_URL,
				content: {
					url: serverUrl
				},	
				handleAs: "json",
				
				failOk: true

			}, false /* add project id */);
			
			def.addCallback(dojo.hitch(this, function (response){
				this._updateStatus(resultNode, response);
			}));
			def.addErrback(dojo.hitch(this, function (){
				this._updateStatus(resultNode, null);
			}));
		},
		
		_fillRow: function(rowNum, createToolbar) {
			var j, text;
			var serverDesc = this._servers[rowNum];
			var rowNode = this._table.tbody.rows[rowNum];
			
			var cellsToAdd = this._columns.length - rowNode.cells.length; 
			for (j = 0; j < cellsToAdd; ++j) {
				td = dojo.create("td");
				rowNode.appendChild(td);
			}
			
			for(j = 0; j < this._columns.length; j++){
				 if (this._columns[j].id === COLUMN_ID_STATUS) {
					this._checkStatus(rowNode.cells[j], serverDesc[COLUMN_ID_URL]); 
				 } else if (this._columns[j].id !== COLUMN_ID_ACTIONS) {
					 var content = serverDesc[this._columns[j].id];
					 rowNode.cells[j].innerHTML = content !== undefined ? jazz.util.html.escape(content) : ""; 
				 } else if (createToolbar) {
					 toolbar = new uml.rmps.rendering.RenderingAdminPageToolbar();
					 td.appendChild(toolbar.domNode);
					 toolbar.addButton(new jazz.ui.toolbar.Button({
							imgSrc: dojo.moduleUrl("rmps.app","graphics/icons/edit.gif").toString(),
							onClick: dojo.hitch(this, this._editServer),
							rowNumber: rowNum,
							title: messages.editServerToolbarHint
					 }));
					 
					 toolbar.addButton(new jazz.ui.toolbar.Button({
							imgSrc: dojo.moduleUrl("rmps.app","graphics/icons/remove.gif").toString(),
							onClick: dojo.hitch(this, this._deleteServer),
							rowNumber: rowNum,
							title: messages.deleteServerToolbarHint
						}));
				 
				 }
				
			 }

		},
		
		_addRow: function(ind) {
			var tr = dojo.create("tr");
			this._table.tbody.appendChild(tr);
			this._fillRow(ind, true);
		},
				
		_createTable: function() {
			var toolbar, editBtn, deleteBtn;
			this._table = new jazz.ui.StyledTable({
				columns: this._columns,
				showHover: true
			});
			
			this._pageNode.appendChild(this._table.domNode);
			this._table.domNode.style.width = "100%";
			dojo.addClass(this._table.domNode, "rmps-app-admin-rendering-Table");
			
			
			for (var i = 0; i < this._servers.length; i++) {
				this._addRow(i);
			}
		},
		
		_editServer: function(e) {
			dojo.stopEvent(e);
			var button = e.currentTarget;
			var dialog = new uml.rmps.rendering.RenderingServerEditDialog({
				columns: this._columns,
				serverDesc: this._servers[button.rowNumber],
				onOK: dojo.hitch(this, function(result){
					this._putRenderingServers();	
					this._fillRow(button.rowNumber, false);
				})
			});
			dialog.show();
		},
		
		_deleteServer: function(e) {
			dojo.stopEvent(e);
			var button = e.currentTarget;
			if (confirm(dojo.string.substitute(messages.deleteConfirmationDialog, [this._servers[button.rowNumber].Name]))) {
				this._servers.splice(button.rowNumber, 1);
				this._putRenderingServers();
				this.refresh(/* onlyTable */ true);
			}
		},
		
		_getRenderingServers: function() {
			
			var def = rmps.core.client.xhrGet({
				url: RENDERING_SERVERS_URL,
				handleAs: 'json',
				headers: { "Accept" : "application/json" }
			}, false /* add project id */);
			
			def.addCallback(dojo.hitch(this, function(servers){
				this._servers = servers;
			}));
			
			return def;
		},
		
		
		_putRenderingServers: function() {
			var def = rmps.core.client.xhrPut({
				url: RENDERING_SERVERS_URL,
				handleAs: 'json',
				headers: { "Accept" : "application/json",
						   "Content-Type" : "application/json"},
				putData: dojo.toJson(this._servers)		   
			}, false /* add project id */);
			
			return def;
		},
		
		_createTopControls: function() {
			var toolbar = new jazz.ui.Toolbar();
			this._topControlsNode.appendChild(toolbar.domNode);
			toolbar.addChild(new jazz.ui.toolbar.Button({
				imgSrc: dojo.moduleUrl("rmps.app","graphics/icons/refresh.gif").toString(),
				onClick: dojo.hitch(this, function() { this.refresh(false /*onlyTable*/); }),
				title: messages.refreshToolbarHint
			}));
			
			toolbar.addChild(new jazz.ui.toolbar.Separator());
			
			toolbar.addChild(new jazz.ui.toolbar.Button({
				label: messages.newServer,
				imgSrc: dojo.moduleUrl("rmps.app","graphics/icons/modelexplorer.gif").toString(),
				onClick: dojo.hitch(this, this._addNewServer),
				title:  messages.newServerToolbarHint
			}));
		},
		
		
		_addNewServer: function() {
			var dialog = new uml.rmps.rendering.RenderingServerEditDialog({
				columns: this._columns,
				serverDesc: {},
				onOK: dojo.hitch(this, function(result){
					this._servers.push(result);
					this._putRenderingServers();
					this._addRow(this._servers.length - 1);
				}),
				_isNew: true
			});
			dialog.show();
		},
		
		refresh: function(onlyTable) {
			if (this._table) {
				this._table.destroy();
			}
			if (!onlyTable) {
				this._servers = [];
				this._getRenderingServers().then(dojo.hitch(this, this._createTable));
			} else {
				this._createTable();
			}
		}
			
	});

}());
