/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2013. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/

(function(){
	dojo.provide("uml.rmps.rendering.RenderingServerEditDialog");
	
	dojo.require("dijit._Widget");
	dojo.require("dijit._Templated");
	dojo.require("dijit.form.TextBox");
	dojo.require("dijit.form.CheckBox");
	
	dojo.requireLocalization("uml.rmps.rendering", "RenderingAdminPageMessages");
	
	
	dojo.declare("uml.rmps.rendering.RenderingServerEditDialog", [dijit._Widget, dijit._Templated], {
		templatePath: dojo.moduleUrl("uml.rmps", "rendering/templates/RenderingServerEditDialog.html"),
		widgetsInTemplate: true,
		
		constructor: function() {
			this.messages = dojo.i18n.getLocalization("uml.rmps.rendering", "RenderingAdminPageMessages");
			this.textBoxes = {};
		},
		
		postCreate: function() {
			var tr, td1, td2, td3, edit, name, id;
			for (var i=0; i < this.columns.length; ++i) {
				if (this.columns[i].isEditable) {
					tr = dojo.create("tr");
					td1 = dojo.create("td", { align: "right"});
					name = this.columns[i].label + " :";
					tr.appendChild(td1);
					tr.appendChild(dojo.create("td"));
					td2 = dojo.create("td");
					tr.appendChild(td2);
					if (this.columns[i].checkbox) {
						id = "editDomainExtensionDialogCheckBox_" + name;
						edit = new dijit.form.CheckBox({
					        checked: this.serverDesc[this.columns[i].id],
					        id: id
						});
						var label = dojo.create("label", {
							"for" : id,
							innerHTML : name
						});
						td1.appendChild(label); 
					} else {
						edit = new dijit.form.TextBox({
						    value: this.serverDesc[this.columns[i].id]
						 });
						dojo.style(edit.domNode, "width", "100%");
						td1.innerHTML = name;
					}
					this.textBoxes[this.columns[i].id] = edit;
					
					td2.appendChild(edit.domNode);
					this.controlsTableNode.appendChild(tr);
				}
			}
		},
		
		show: function () {
			this.dialog = new jazz.ui.Dialog({
				contentNode: this.containerNode,
				primaryTitle: this._isNew ? this.messages.newServerDialogTitle : this.messages.editServerDialogTitle,
				width: "600px"
			});
		},
		
		_closeDialog: function() {
			if (this.dialog) {
				this.dialog.close();
				this.dialog.destroyRecursive();
			}
		},
		
		_onOK: function() {
			var control;
			for (var attr in this.textBoxes) {
				if (this.textBoxes.hasOwnProperty(attr)) {
					control = this.textBoxes[attr];
					this.serverDesc[attr] = this._getValue(control);  
				}
			}
			
			
			if (this.onOK && dojo.isFunction(this.onOK)){
				this.onOK(this.serverDesc);
			}
			this._closeDialog();
		},
		
		_onCancel: function() {
			this._closeDialog();
		},
		
		_getValue: function(control) {
			if (control.hasOwnProperty("checked")) {
				return control.checked ? true : false;
			} else {
				return control.value;
			}
			
		}
	});
	
	
	
}());